function [ class1, class2 ] = ClassifyEdges( M )
% Mag format:
% G(X,Y) == 0 : no edge
% G(X,Y) == 1 : X*-oY (cannot be in MAGs)
% G(X,Y) == 2 : X*->Y
% G(X,Y) == 3 : X*--Y

n = length(M);
class1 = zeros(n);
class2 = zeros(n);

M_dir = M == 2 & M' == 3;
closure = transitiveClosureSparse_mex(sparse(M_dir));
visible_edges = findVisibleEdges(M);

% Directed edges
total_dir = sum(M_dir(:));
[X Y] = find(M_dir);

% For each directed edge:
for i = 1:total_dir
    if(visible_edges(X(i),Y(i)))
        class1(X(i),Y(i)) = 1;
    else
        class1(X(i),Y(i)) = 2;
    end
end

% For each directed edge X -> Y:
for i = 1:total_dir
    if(class1(X(i),Y(i)) == 1)
        % Edge is possibly-indirect
        if(hasPossiblyInducingPath(X(i),Y(i),M_dir, class1))
            class2(X(i),Y(i)) = 2;
        % Edge is direct-causal
        else
            class2(X(i),Y(i)) = 1;
        end
    elseif(class1(X(i),Y(i)) == 2)
        if(any(closure(M_dir(X(i),:),Y(i))))
            class2(X(i),Y(i)) = 2;
        else
            class2(X(i),Y(i)) = 1;
        end
    end
end

end

function res = hasPossiblyInducingPath(X,Y,M,class1)

res = false;
for i = 1:length(M)
   if(M(X,i) && M(i,Y) && class1(i,Y) == 2)
       res = true;
       return;
   end
end
end

function [ visible ] = findVisibleEdges( M )

nnodes = size(M,1);

M_dir = M == 2 & M' == 3;
total_dir = sum(M_dir(:));

M_bidir = M == 2 & M' == 2;

M_e = M_dir | M_bidir;

visible = false(nnodes);
Q = zeros(total_dir,2);
curq = 0;

[X Y] = find(M_dir);

for i = 1:total_dir
    for j = 1:nnodes
        if M_e(j,X(i)) && ~M_e(j,Y(i))
            visible(X(i),Y(i)) = 1;
            curq = curq + 1;
            Q(curq,1) = X(i);
            Q(curq,2) = Y(i);
            break;
        end
    end
end

while curq
   X = Q(curq,1);
   Y = Q(curq,2);
   curq = curq - 1;

   W = find(M_bidir(:,X) & M_dir(:,Y) & ~visible(:,Y));
   len = length(W);
   if(len > 0)
       visible(W,Y) = 1;
       Q(curq+1:curq+len,1) = W;
       Q(curq+1:curq+len,2) = Y;
       curq = curq + len;
   end
end

end

